***************************************************************************
Author: L. Pitonakova (http://lenkaspace.net)

License: GNU General Public License. Please credit me when using my work.

***************************************************************************


An ARGoS simulation of robot swarm foraging using various robot controllers and environments. 

This code has been used in various papers (see References) and thus allows to run different types of simulations. See "Specifying experiment parameters" below.

All robots start in the base, placed in the centre of the arena. The base is divided into three sections: an interior circular resting bay with an annular observation bay around it and an annular unloading bay around that. A light source, placed above the centre of the base, is used by the robots as a reference for navigation towards and away from the base centre.

There are two foraging missions possible:
  - In central-place foraging [2,3,4,5], robots search for worksites (green cylinders, also referred to as "tasks" or "deposits" in the code) and carry resources back to the unloading bay. Robots in the bee-inspired swarm [1,2,3,4,5] then use the observation bay to recruit other robots nearby.
  - In basic foraging [2,3], robots do not bring resources back to the base and consume the deposits while near them over time.

Each worksite can require N robots to work on it, meaning that no resource can be extracted unless at least N robots are nearby.

Also, each worksite has a "quality" associated with it, which is the amount of resource a robot gets by extracting one unit of volume from the worksite (see [4]).

There are three different robot controllers:
  - Solitary [1,2]: Robots do not communicate
  - Local broadcaster [1,2,3]: Robots recruit each other near worksites
  - Bee [1,2,3,4,5]: Robots recruit each other in the base.




FOLDER STRUCTURE
=======================================

  - cmake: CMake files
  - controllers: Libraries for various robot controllers
  - loop_functions: Libraries for representing and drawing the environment



BUILDING AND RUNNING SIMULATION CODE
=======================================

Built using ARGoS3 version 3.0.0-beta51. Different ARGoS versions may produce compiler errors.

First, invoke cmake with

```
./build.sh -m
```

Build with

```
./build.sh -c
```

A simulation can be run using an .argos file, e.g.

```
./build.sh -r _experiments/taskDiscovery.argos
```


SPECIFYING EXPERIMENT PARAMETERS
=======================================

Experiment parameters can be specified by changing the .argos file (use _experiments\taskDiscovery.argos as a template).


ROBOT CONTROLLERS
---------------------------------------

Each robot controller has its own section in the <controllers> node.
The <baseParameters> node specifies common parameters used by the BaseRobot class.
The <parameters> node specifies controller-specific parameters used by the controller's respective class.

The robot controller parameters and their optimisation are described in detail in [2].

The base parameters are
  - linuxBased: Deprecated. Use "1".
  - cartCapacity: The number of units of resource a robot can carry to the base during central-place foraging
  - loadTimePerVolumeUnit: The number of update ticks it takes to load one unit of resource
  - unloadTimePerVolumeUnit: The number of update ticks it takes to unload one unit of resource in the base
  - neighbourhoodSearchRadius: Distance in meters from presumed worksite location that a robot searches when no worksite can be found
  - neighbourhoodSearchTime: Time in seconds that a robot searches for a worksite when it is not found at a presumed location
  - expectedArenaDiameter: The maximum distance in meters that a robot travels away from the base when scouting
  - useOpportunism: When true, preferential foraging is turned on (as in [3])
  - opportunismThreshold: The difference in worksite qualities that determines whether an opportunistic robot switches to a new worksite (as in [3])
  - useRelU: Deprecated.


Solitary controller parameters are:
  - maxTeamSizeUnchangedTime: The amount of time in seconds that a robot waits near a worksite when multiple robots are required to work
			      on the worksite. Each new worker arrival resets the waiting time counter.

Local broadcaster parameters are:
  - maxTeamSizeUnchangedTime: The amount of time in seconds that a robot waits near a worksite when multiple robots are required to work
			      on the worksite. Each new worker arrival resets the waiting time counter.
  - signalDistance: Recruitment signal range in centimeters


Bee controller parameters are:
  - maxTeamSizeUnchangedTime: The amount of time in seconds that a robot waits near a worksite when multiple robots are required to work
			      on the worksite. Each new worker arrival resets the waiting time counter.
  - signalDistance: Recruitment signal range in centimeters
  - maxWaggleDanceTime: Maximum waggle dance (recruitment) time in seconds. Actual time is dependent on worksite quality.
  - scoutingProb: The probability with which a robot leaves the base to scout
  - maxScoutingTime: Maximum scouting time in seconds. When expired, a scouting robot returns to the base.


Other parameters defined in the XML setup the sensors and actuators and can mostly be left untouched. See ARGoS help for how to setup the various robot components, for example to add noise to them.



ARENA
---------------------------------------

The arena can be setup using the <arena> node.

The XML defines four walls that surrounds the arena and a light in the middle.

Robots are then placed into the environment:
  - The number of robots can be set using the "quantity" attribute of the <entity> node that distributes <foot-bot> entities. 
  - The controller type can be set using the "config" property of the <foot-bot> -> <controller> node. Can be "solitary, "localBroadcaster" or "beeRecruiter"

Worksites are then placed into the environment:
  - The number of robots can be set using the "quantity" attribute of the <entity> node that distributes <cylinder> entities.
  - Worksite distribution and properties are set via the simulation parameters, see below.



SIMULATION PARAMETERS
---------------------------------------

The simulation parameters are set via the <loop-functions> -> <foraging> node attributes:
  - defaultTaskType: Can be "COLLECT" for central-place foraging or "PERFORM" for general foraging
  - taskGradientRadius: The gradient radius in meters that is drawn around each worksite, helping robots to find the worksite
  - taskMinDistanceFromBase, taskMaxDistanceFromBase: Worksites are placed randomly between taskMinDistanceFromBase and taskMaxDistanceFromBase metres from the base centre.
  - outputFileNamePrefix: See the Output Logs section below.
  - evenlySpacedTasks: If true, worksites will be spread out as evenly as possible around the base. Used e.g. to generate the "Heap" environments from [2,3,4]
  - minRobotsForTask, maxRobotsForTask: Each worksite will require a random number between minRobotsForTask and maxRobotsForTask robots to work on it before resource can be extracted.

  - taskDisappearRate: If > 0, worksites will disappear each taskDisappearRate seconds and new worksites will appear randomly in the environment. Used in [2].

  - taskRenewabilityThreshold: If >= 0, worksites, volumes of which have reached less than taskRenewabilityThreshold percent, will have their volume replenished and will randomly change their position.

  - taskAppearRate, taskAppearTime, taskAppearStartingPercentage: If taskAppearRate > 0, worksites only taskAppearStartingPercentage percent of worksites are immediately available. The rest of the worksites appear one by one each taskAppearRate seconds, starting at time taskAppearTime.
  
  - group1Quality ... group4Quality: Quality (0-1) of 4 quality groups, that will be randomly assigned to worksites. Works only for up to 4 worksites. Additional worksite get assigned quality of group 1. Used in [4].
  - taskQualityChangeRate: Rate in seconds at which worksite qualities are randomly reassigned. When this happens, worksite volumes are replenished. Used in [4].




OUTPUT LOGS
---------------------------------------

Two log files are produced during the simulation:
 - global: Second-by-second record of how much resource has been collected
 - events: Various events that occurred, e.g. worksites being added or depleted

Use the "outputFileNamePrefix" simulation parameter to specify the path and prefix of log files, either using an absolute path or path relative to the ./build.sh file (i.e. the folder that contains the simulation code).

The prefix is useful e.g. when using bash scripts to generate different simulation run settings.




REFERENCES
=======================================

[1] Pitonakova, L., Crowder, R., & Bullock, S. (2018). Information exchange design patterns for robot swarm foraging and their application in robot control algorithms. Frontiers in Robotics and AI. http://doi.org/10.3389/frobt.2018.00047

[2] Pitonakova, L., Crowder, R., & Bullock, S. (2018). The Information-Cost-Reward framework for understanding robot swarm foraging. Swarm Intelligence, 12(1), 71–96. http://doi.org/10.1007/s11721-017-0148-3

[3] Pitonakova, L., Crowder, R. & Bullock, S. (2018). ﻿The Importance of Information Flow Regulation in Preferentially Foraging Robot Swarms. Proceedings of the Eleventh International Conference on Swarm Intelligence (ANTS 2018), Springer.

[4] Pitonakova, L., Crowder, R., & Bullock, S. (2016). Information flow principles for plasticity in foraging robot swarms. Swarm Intelligence, 10(1), 33–63.
https://link.springer.com/article/10.1007/s11721-016-0118-1

[5] Pitonakova, L., Crowder, R., & Bullock, S. (2016). Task allocation in foraging robot swarms: The role of information sharing. In G. Gershenson, T. Froese, J. M. Siqueiros, W. Aguilar, E. J. Izquierdo, & H. Sayama (Eds.), Proceedings of the Fifteenth International Conference on the Synthesis and Simulation of Living Systems (ALIFE XV) (pp. 306–313). Cambridge, MA: MIT Press.

