/* ================================================================
 * ================================================================
 *
 * A "solitary" robot controller.
 * Robots do not communicate.
 *
 * Author: L. Pitonakova (http://lenkaspace.net)
 * License: GNU General Public License. Please credit me when using my work.
 *
 * Update loop of the finite-state machine:
 *
 * 	SCOUT		----->	(rT=RT) -----> 		LOAD
 * 						(else) ------> 		WAIR NEAR TASK
 *
 * 	LOAD 		----->	(task done) ----->	SCOUT
 *
 * 	WAIT N.T.	----->	(rT=RT)		----->	LOAD
 * 						(time up)	----->	SCOUT
 *
 *
 * This controller has been used in, e.g.
 * Pitonakova, L., Crowder, R., & Bullock, S. (2018). The Information-Cost-Reward framework for understanding robot swarm foraging. Swarm Intelligence, 12(1), 71–96. http://doi.org/10.1007/s11721-017-0148-3
 *
 * ================================================================
 * ================================================================
 */
 
 
#ifndef SOLITARY_H
#define SOLITARY_H

#include "baseRobot.h"

using namespace argos;

class Solitary : public BaseRobot {

public:
	struct Parameters {
	   uint maxTeamSizeUnchangedTime; // how long a robot waits by a task if it hasn't recruited any other robot to it (in seconds)

	   Parameters() {};
	   void Init(TConfigurationNode& t_node);
   };

	Solitary();
	void Init(TConfigurationNode& t_node);
	void Reset();

	void ControlStep();
	void ProcessTaskReached();
	Real ProcessTask(TASK_TYPE taskType_, int taskId_,  Real processedAmount_, Real taskValue_, Real taskTotalReward_, uint numOfRobotsRequired_);
	void OnUnloadResourceDone();


private:
	void OnTaskScouted();

	void BroadcastTaskCommitment();
	void FindOtherTaskBroadcasters();
	void ReturnToBase();
	void GoToTask();


	Parameters parameters;

	//-- behaviour
	uint timeSinceLastRobotArrivedToTask;
	uint lastNumOfRobotsAtTask;

	//-- communication
	std::vector<uint>encounteredIdsAtTask;
};

#endif



